#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['tankobon', 'tankobon._parsers']

package_data = \
{'': ['*'],
 'tankobon': ['.ropeproject/*'],
 'tankobon._parsers': ['.ropeproject/*']}

install_requires = \
['beautifulsoup4>=4.9.1',
 'click>=7.1.2',
 'coloredlogs>=14.0',
 'fake-useragent>=0.1.11',
 'filetype>=1.0.7',
 'requests>=2.24.0']

extras_require = \
{'gui': ['PySide6>=6.0.1'],
 'pdf': ['fpdf>=1.7.2', 'imagesize>=1.2.0', 'natsort>=7.1.0']}

entry_points = \
{'console_scripts': ['tankobon = tankobon.cli:cli']}

setup(name='tankobon',
      version='5.0.0b1',
      description='Yet another manga scraper and downloader',
      author='Ong Yong Xin',
      author_email='ongyongxin2020+github@gmail.com',
      url='https://github.com/ongyx/tankobon',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.6',
     )
